@@ -1,16 +1,21 @@
 from datetime import timedelta
-import time
 import logging
+import os
 import re
 import requests
+import time
 
-from django.db import models
 from django.conf import settings
+from django.db import models
 from django.utils import timezone
 from django.utils.crypto import get_random_string
 from django.utils.http import same_origin
-from django.contrib.auth.models import User
 from django.utils.translation import ugettext_lazy as _
+try:
+    from django.contrib.auth import get_user_model
+    User = get_user_model()
+except ImportError: # Django version < 1.5
+    from django.contrib.auth.models import User
 
 from mama_cas.exceptions import InvalidRequestError
 from mama_cas.exceptions import InvalidTicketError
@@ -20,6 +25,7 @@ from mama_cas.exceptions import BadPGTError
 from mama_cas.utils import add_query_params
 from mama_cas.utils import is_scheme_https
 from mama_cas.utils import clean_service_url
+from mama_cas.utils import is_valid_service_url
 
 
 logger = logging.getLogger(__name__)
@@ -88,7 +94,7 @@ class TicketManager(models.Manager):
         if not service:
             raise InvalidRequestError("No service identifier provided")
 
-        if not self.is_valid_service_url(service):
+        if not is_valid_service_url(service):
             raise InvalidServiceError("Service %s is not a valid %s URL" %
                                       (service, title))
 
@@ -103,22 +109,6 @@ class TicketManager(models.Manager):
         logger.debug("Validated %s %s" % (title, ticket))
         return t
 
-    def is_valid_service_url(self, url):
-        """
-        Check the provided service URL against the configured list of valid
-        service URLs. If the service URL matches at least one valid service,
-        return ``True``, otherwise return ``False``. If no valid service URLs
-        are configured, return ``True``.
-        """
-        valid_services = map(re.compile,
-                             getattr(settings, 'MAMA_CAS_VALID_SERVICES', ()))
-        if len(valid_services) == 0:
-            return True
-        for service in valid_services:
-            if service.match(url):
-                return True
-        return False
-
     def delete_invalid_tickets(self):
         """
         Iterate over all ``Ticket``s and delete all consumed or expired
@@ -293,7 +283,8 @@ class ProxyGrantingTicketManager(TicketManager):
         # Connect to proxy callback URL, checking the SSL certificate
         pgturl = add_query_params(pgturl, {'pgtId': pgtid, 'pgtIou': pgtiou})
         try:
-            r = requests.get(pgturl, verify=True)
+            verify = os.environ.get('REQUESTS_CA_BUNDLE', True)
+            r = requests.get(pgturl, verify=verify)
         except (requests.exceptions.ConnectionError,
                 requests.exceptions.SSLError) as e:
             raise InternalError("%s" % e)
@@ -329,7 +320,7 @@ class ProxyGrantingTicketManager(TicketManager):
             raise InvalidTicketError("%s %s has already been used" %
                                      (title, ticket))
 
-        if not self.is_valid_service_url(service):
+        if not is_valid_service_url(service):
             raise InvalidServiceError("Service %s is not a valid %s URL" %
                                       (service, title))
 
