@@ -1,26 +1,27 @@
 import logging
 
+from django.contrib import messages
+from django.contrib.auth import login
+from django.core.urlresolvers import reverse
 from django.http import HttpResponse
 from django.shortcuts import redirect
 from django.utils.http import urlquote_plus
-from django.core.urlresolvers import reverse
-from django.contrib import messages
+from django.utils.translation import ugettext as _
 from django.views.generic import FormView
 from django.views.generic import TemplateView
 from django.views.generic import View
-from django.contrib import auth
-from django.utils.translation import ugettext as _
 
 from mama_cas.forms import LoginForm
 from mama_cas.forms import WarnForm
-from mama_cas.models import ServiceTicket
-from mama_cas.models import ProxyTicket
-from mama_cas.utils import add_query_params
-from mama_cas.mixins import NeverCacheMixin
-from mama_cas.mixins import LoginRequiredMixin
-from mama_cas.mixins import ValidateTicketMixin
 from mama_cas.mixins import CustomAttributesMixin
+from mama_cas.mixins import LoginRequiredMixin
 from mama_cas.mixins import LogoutUserMixin
+from mama_cas.mixins import NeverCacheMixin
+from mama_cas.mixins import ValidateTicketMixin
+from mama_cas.models import ProxyTicket
+from mama_cas.models import ServiceTicket
+from mama_cas.utils import add_query_params
+from mama_cas.utils import is_valid_service_url
 
 
 logger = logging.getLogger(__name__)
@@ -30,7 +31,7 @@ class LoginView(NeverCacheMixin, LogoutUserMixin, FormView):
     """
     (2.1 and 2.2) Credential requestor and acceptor.
 
-    This URI operates in two modes: a credential requestor when a GET request
+    This view operates as a credential requestor when a GET request
     is received, and a credential acceptor for POST requests.
     """
     template_name = 'mama_cas/login.html'
@@ -38,18 +39,19 @@ class LoginView(NeverCacheMixin, LogoutUserMixin, FormView):
 
     def get(self, request, *args, **kwargs):
         """
-        As a credential requestor, /login accepts up to three optional
+        (2.1) As a credential requestor, /login accepts three optional
         parameters:
 
         1. ``service``: the identifier of the application the client is
-           accessing. In most cases this will be a URL.
-        2. ``renew``: requires a client to present credentials regardless of
-           any existing single sign-on session. If set, its value should be
-           "true".
-        3. ``gateway``: causes the client to not be prompted for credentials.
-           If a single sign-on session already exists, the user will be logged
-           in. Otherwise, the user is simply forwarded to the service, if
-           specified. If set, its value should be "true".
+           accessing. We assume this identifier to be a URL.
+        2. ``renew``: requires a client to present credentials
+           regardless of any existing single sign-on session. If set,
+           its value should be ``true``.
+        3. ``gateway``: causes the client to not be prompted for
+           credentials. If a single sign-on session already exists, the
+           user will be logged in. Otherwise, the user is simply
+           forwarded to the service, if specified. If set, its value
+           should be ``true``.
         """
         service = request.GET.get('service')
         renew = request.GET.get('renew')
@@ -88,40 +90,42 @@ class LoginView(NeverCacheMixin, LogoutUserMixin, FormView):
                 logger.debug("Redirecting to %s" % service)
                 return redirect(service)
             else:
-                messages.success(request,
-                                 _("You are logged in as %s") % request.user)
+                msg = _("You are logged in as %s") % request.user
+                messages.success(request, msg)
         return super(LoginView, self).get(request, *args, **kwargs)
 
     def warn_user(self):
         """
-        Returns ``True`` if the ``warn`` parameter is set in the current
-        session. Otherwise, returns ``False``.
+        Returns ``True`` if the ``warn`` parameter is set in the
+        current session. Otherwise, returns ``False``.
         """
         return self.request.session.get('warn', False)
 
     def form_valid(self, form):
         """
-        As a credential acceptor, /login takes two required parameters:
+        (2.2) As a credential acceptor, /login requires two parameters:
 
         1. ``username``: the username provided by the client
         2. ``password``: the password provided by the client
 
-        If authentication is successful, the user is logged in which creates
-        the single sign-on session. If a service is provided, a corresponding
-        ``ServiceTicket`` is created, and the user is redirected to the
-        service URL. If no service is provided, the user is redirected back
-        to the login page with a message indicating a successful login.
+        If authentication is successful, the single sign-on session is
+        created. If a service is provided, a ``ServiceTicket`` is
+        created and the client is redirected to the service URL with
+        the ``ServiceTicket`` included. If no service is provided, the
+        login page is redisplayed with a message indicating a
+        successful login.
 
         If authentication fails, the login form is redisplayed with an
-        appropriate error message displayed indicating the reason for failure.
+        error message describing the reason for failure.
 
-        The credential acceptor also accepts one optional parameter:
+        The credential acceptor accepts one optional parameter:
 
-        1. ``warn``: causes user input to be required whenever an
-           authentication attempt occurs within the single sign-on session.
+        1. ``warn``: causes the user to be prompted when successive
+           authentication attempts occur within the single sign-on
+           session.
         """
-        auth.login(self.request, form.user)
-        logger.info("Single sign-on session started for %s" % self.request.user)
+        login(self.request, form.user)
+        logger.info("Single sign-on session started for %s" % form.user)
 
         if form.cleaned_data.get('warn'):
             self.request.session['warn'] = True
@@ -144,18 +148,13 @@ class LoginView(NeverCacheMixin, LogoutUserMixin, FormView):
 
 class WarnView(NeverCacheMixin, LoginRequiredMixin, FormView):
     """
-    (2.2.1) Disable transparent authentication by displaying a page indicating
-    that authentication is taking place. The user can then choose to continue
-    or cancel the authentication attempt.
+    (2.2.1) Disables transparent authentication by informing the user
+    that service authentication is taking place. The user can choose
+    to continue or cancel the authentication attempt.
     """
     template_name = 'mama_cas/warn.html'
     form_class = WarnForm
 
-    def get(self, request, *args, **kwargs):
-        if not request.GET.get('service'):
-            return redirect(reverse('cas_login'))
-        return super(WarnView, self).get(request, *args, **kwargs)
-
     def form_valid(self, form):
         service = form.cleaned_data.get('service')
         gateway = form.cleaned_data.get('gateway')
@@ -183,22 +182,20 @@ class LogoutView(NeverCacheMixin, LogoutUserMixin, View):
     """
     (2.3) End a client's single sign-on session.
 
-    When this URI is accessed, any current single sign-on session is
+    When this view is accessed, an existing single sign-on session is
     ended, requiring a new single sign-on session to be established
     for future authentication attempts.
 
-    If ``url`` is specified, it will be displayed to the user as a recommended
-    link to follow.
+    If ``url`` is specified, it will be displayed to the user as a
+    recommended link to follow.
     """
     def get(self, request, *args, **kwargs):
         logger.debug("Logout request received for %s" % request.user)
         self.logout_user(request)
         url = request.GET.get('url', None)
-        if url:
-            messages.success(request,
-                             _("The application provided this link to follow: "
-                             "<a href=\"%(url)s\">%(url)s</a>") % {'url': url},
-                             extra_tags='safe')
+        if url and is_valid_service_url(url):
+            msg = _("The application provided this link to follow: %s") % url
+            messages.success(request, msg)
         return redirect(reverse('cas_login'))
 
 
@@ -206,14 +203,14 @@ class ValidateView(NeverCacheMixin, ValidateTicketMixin, View):
     """
     (2.4) Check the validity of a service ticket. [CAS 1.0]
 
-    When both ``service`` and ``ticket`` are provided, this URI responds with
-    a ``ServiceTicket`` validation success or failure. Whether or not the
-    validation succeeds, the ``ServiceTicket`` is consumed, rendering it
-    invalid for future authentication attempts.
+    When both ``service`` and ``ticket`` are provided, this view
+    responds with a ``ServiceTicket`` validation success or failure.
+    Whether or not the validation succeeds, the ``ServiceTicket`` is
+    consumed, rendering it invalid for future authentication attempts.
 
     If ``renew`` is specified, validation will only succeed if the
-    ``ServiceTicket`` was issued from the presentation of the user's primary
-    credentials (i.e. not from an existing single sign-on session).
+    ``ServiceTicket`` was issued from the presentation of the user's
+    primary credentials, not from an existing single sign-on session.
     """
     def get(self, request, *args, **kwargs):
         st, pgt, error = self.validate_service_ticket(request)
@@ -229,16 +226,17 @@ class ServiceValidateView(NeverCacheMixin, ValidateTicketMixin,
     """
     (2.5) Check the validity of a service ticket. [CAS 2.0]
 
-    When both ``service`` and ``ticket`` are provided, this URI responds with
-    an XML-fragment response indicating a ``ServiceTicket`` validation success
-    or failure. Whether or not the validation succeeds, the ``ServiceTicket``
-    is consumed, rendering it invalid for future authentication attempts.
+    When both ``service`` and ``ticket`` are provided, this view
+    responds with an XML-fragment response indicating a
+    ``ServiceTicket`` validation success or failure. Whether or not
+    validation succeeds, the ticket is consumed, rendering it invalid
+    for future authentication attempts.
 
     If ``renew`` is specified, validation will only succeed if the
-    ``ServiceTicket`` was issued from the presentation of the user's primary
-    credentials (i.e. not from an existing single sign-on session).
+    ``ServiceTicket`` was issued from the presentation of the user's
+    primary credentials, not from an existing single sign-on session.
 
-    If ``pgtUrl`` is specified, the response will also include a
+    If ``pgtUrl`` is specified, the response will include a
     ``ProxyGrantingTicket`` if the proxy callback URL has a valid SSL
     certificate and responds with a successful HTTP status code.
     """
@@ -255,20 +253,20 @@ class ServiceValidateView(NeverCacheMixin, ValidateTicketMixin,
 class ProxyValidateView(NeverCacheMixin, ValidateTicketMixin,
                         CustomAttributesMixin, TemplateView):
     """
-    (2.6) Check the validity of a service ticket, and additionally
-    validate proxy tickets. [CAS 2.0]
+    (2.6) Perform the same validation tasks as ServiceValidateView and
+    additionally validate proxy tickets. [CAS 2.0]
 
-    When both ``service`` and ``ticket`` are provided, this URI responds with
-    an XML-fragment response indicating a ``ProxyTicket`` or ``ServiceTicket``
-    validation success or failure. Whether or not the validation succeeds, the
-    ``ProxyTicket`` or ``ServiceTicket`` is consumed, rendering it invalid for
-    future authentication attempts.
+    When both ``service`` and ``ticket`` are provided, this view
+    responds with an XML-fragment response indicating a ``ProxyTicket``
+    or ``ServiceTicket`` validation success or failure. Whether or not
+    validation succeeds, the ticket is consumed, rendering it invalid
+    for future authentication attempts.
 
     If ``renew`` is specified, validation will only succeed if the
-    ``ServiceTicket`` was issued from the presentation of the user's primary
-    credentials (i.e. not from an existing single sign-on session).
+    ``ServiceTicket`` was issued from the presentation of the user's
+    primary credentials, not from an existing single sign-on session.
 
-    If ``pgtUrl`` is specified, the response will also include a
+    If ``pgtUrl`` is specified, the response will include a
     ``ProxyGrantingTicket`` if the proxy callback URL has a valid SSL
     certificate and responds with a successful HTTP status code.
     """
@@ -296,10 +294,11 @@ class ProxyView(NeverCacheMixin, ValidateTicketMixin, TemplateView):
     (2.7) Provide proxy tickets to services that have acquired proxy-
     granting tickets. [CAS 2.0]
 
-    When both ``pgt`` and ``targetService`` are specified, this URI responds
-    with an XML-fragment response indicating a ``ProxyGrantingTicket``
-    validation success or failure. If validation succeeds, a ``ProxyTicket``
-    will be created and included in the response.
+    When both ``pgt`` and ``targetService`` are specified, this view
+    responds with an XML-fragment response indicating a
+    ``ProxyGrantingTicket`` validation success or failure. If
+    validation succeeds, a ``ProxyTicket`` will be created and included
+    in the response.
     """
     template_name = 'mama_cas/proxy.xml'
 
